==> racer-0.0.4.tgz: package/lib/adapters/Mongo.js <==
var CONNECTED, CONNECTING, Collection, DISCONNECTED, DISCONNECTING, EventEmitter, MongoAdapter, MongoCollection, ObjectId, fn, mongo, name, url, _fn, _ref;
var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; }, __slice = Array.prototype.slice;
url = require('url');
mongo = require('mongodb');
EventEmitter = require('events').EventEmitter;
ObjectId = mongo.BSONPure.ObjectID;
ObjectId.toString = function(oid) {
  return oid.toHexString();
};
ObjectId.fromString = function(str) {
  return this.createFromHexString(str);
};
DISCONNECTED = 1;
CONNECTING = 2;
CONNECTED = 3;
DISCONNECTING = 4;
MongoAdapter = module.exports = function(conf) {
  EventEmitter.call(this);
  this._ver = 0;
  if (conf) {
    this._loadConf(conf);
  }
  this._state = DISCONNECTED;
  this._collections = {};
  this._pending = [];
};
MongoAdapter.prototype = {
  _loadConf: function(conf) {
    var uri, _ref, _ref2, _ref3;
    if (typeof conf === 'string') {
      uri = url.parse(conf);
      this._host = uri.hostname;
      this._port = uri.port || 27017;
      this._database = uri.pathname.replace(/\//g, '');
      return _ref3 = (_ref = (_ref2 = uri.auth) != null ? _ref2.split(':') : void 0) != null ? _ref : [], this._user = _ref3[0], this._pass = _ref3[1], _ref3;
    } else {
      return this._host = conf._host, this._port = conf._port, this._database = conf._database, this._user = conf._user, this._pass = conf._pass, conf;
    }
  },
  connect: function(conf, callback) {
    if ('function' === typeof conf) {
      callback = conf;
    } else {
      this._loadConf(conf);
    }
    this._db = new mongo.Db(this._database, new mongo.Server(this._host, this._port));
    this._state = CONNECTING;
    this.emit('connecting');
    return this._db.open(__bind(function(err) {
      var open;
      if (err && callback) {
        return callback(err);
      }
      open = __bind(function() {
        var todo, _i, _len, _ref;
        this._state = CONNECTED;
        this.emit('connected');
        _ref = this._pending;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          todo = _ref[_i];
          this[todo[0]].apply(this, todo[1]);
        }
        return this._pending = [];
      }, this);
      if (this._user && this._pass) {
        return this._db.authenticate(this._user, this._pass, open);
      }
      return open();
    }, this));
  },
  disconnect: function(callback) {
    switch (this._state) {
      case DISCONNECTED:
        return callback(null);
      case CONNECTING:
        return this.once('connected', __bind(function() {
          return this.close(callback);
        }, this));
      case CONNECTED:
        this._state = DISCONNECTING;
        this._db.close();
        this._state = DISCONNECTED;
        this.emit('disconnected');
        if (callback) {
          return callback();
        }
        break;
      case DISCONNECTING:
        return this.once('disconnected', __bind(function() {
          return callback(null);
        }, this));
    }
  },
  flush: function(callback) {
    if (this._state !== CONNECTED) {
      return this._pending.push(['flush', arguments]);
    }
    return this._db.dropDatabase(function(err, done) {
      if (err) {
        throw err;
      }
      return callback();
    });
  },
  set: function(path, val, ver, callback) {
    var collection, delta, id, _ref;
    if (this._state !== CONNECTED) {
      return this._pending.push(['set', arguments]);
    }
    _ref = path.split('.'), collection = _ref[0], id = _ref[1], path = 3 <= _ref.length ? __slice.call(_ref, 2) : [];
    if (path.length) {
      path = path.join('.');
      delta = {
        ver: ver
      };
      delta[path] = val;
      return this._collection(collection).update({
        _id: id
      }, {
        $set: delta
      }, {
        upsert: true,
        safe: true
      }, callback);
    }
    delta = this._flatten(path.join('.'), val);
    delta.ver = ver;
    return this._collection(collection).update({
      _id: id
    }, {
      $set: delta
    }, {
      upsert: true,
      safe: true
    }, callback);
  },
  del: function(path, ver, callback) {
    var collection, id, unset, _ref;
    if (this._state !== CONNECTED) {
      return this._pending.push(['del', arguments]);
    }
    _ref = path.split('.'), collection = _ref[0], id = _ref[1], path = 3 <= _ref.length ? __slice.call(_ref, 2) : [];
    if (path.length) {
      path = path.join('.');
      unset = {};
      unset[path] = 1;
      return this._collection(collection).update({
        _id: id
      }, {
        $unset: unset
      }, {
        safe: true
      }, callback);
    }
    return this._collection(collection).remove({
      _id: id
    }, callback);
  },
  get: function(path, callback) {
    var collection, fields, id, _ref;
    if (this._state !== CONNECTED) {
      return this._pending.push(['get', arguments]);
    }
    _ref = path.split('.'), collection = _ref[0], id = _ref[1], path = 3 <= _ref.length ? __slice.call(_ref, 2) : [];
    if (path.length) {
      path = path.join('.');
      fields = {
        _id: 0,
        ver: 1
      };
      fields[path] = 1;
      return this._collection(collection).findOne({
        _id: id
      }, {
        fields: fields
      }, function(err, doc) {
        if (err) {
          return callback(err);
        }
        return callback(null, doc && doc[path], doc && doc.ver);
      });
    }
    return this._collection(collection).findOne({
      _id: id
    }, function(err, doc) {
      var ver;
      if (err) {
        return callback(err);
      }
      ver = doc && doc.ver;
      if (doc) {
        delete doc.ver;
      }
      return callback(null, doc, ver);
    });
  },
  _collection: function(name) {
    var _base;
    return (_base = this._collections)[name] || (_base[name] = new Collection(name, this._db));
  },
  _genObjectId: function() {
    return ObjectId.toString(new ObjectId);
  },
  _flatten: function(path, obj, delta) {
    var k, nextPath, v;
    if (delta == null) {
      delta = {};
    }
    if (obj.constructor !== Object) {
      delta[path] = obj;
      return delta;
    }
    for (k in obj) {
      v = obj[k];
      nextPath = path ? "" + path + "." + k : k;
      this._flatten(nextPath, v, delta);
    }
    return delta;
  }
};
MongoAdapter.prototype.__proto__ = EventEmitter.prototype;
MongoCollection = mongo.Collection;
Collection = function(name, db) {
  var self;
  self = this;
  self.name = name;
  self.db = db;
  self._pending = [];
  self._ready = false;
  db.collection(name, function(err, collection) {
    if (err) {
      throw err;
    }
    self._ready = true;
    self.collection = collection;
    return self.onReady();
  });
};
Collection.prototype = {
  onReady: function() {
    var todo, _i, _len, _ref;
    _ref = this._pending;
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      todo = _ref[_i];
      this[todo[0]].apply(this, todo[1]);
    }
    return this._pending = [];
  }
};
_ref = MongoCollection.prototype;
_fn = function(name, fn) {
  return Collection.prototype[name] = function() {
    var args, collection;
    collection = this.collection;
    args = arguments;
    if (this._ready) {
      return process.nextTick(function() {
        return collection[name].apply(collection, args);
      });
    } else {
      return this._pending.push([name, arguments]);
    }
  };
};
for (name in _ref) {
  fn = _ref[name];
  _fn(name, fn);
}

==> racer-0.1.3.tgz: package/lib/adapters/Mongo.js <==
var CONNECTED, CONNECTING, Collection, DISCONNECTED, DISCONNECTING, EventEmitter, MongoAdapter, MongoCollection, ObjectId, fn, mongo, name, url, _fn, _ref;
var __slice = Array.prototype.slice;

url = require('url');

mongo = require('mongodb');

EventEmitter = require('events').EventEmitter;

ObjectId = mongo.BSONPure.ObjectID;

ObjectId.toString = function(oid) {
  return oid.toHexString();
};

ObjectId.fromString = function(str) {
  return this.createFromHexString(str);
};

DISCONNECTED = 1;

CONNECTING = 2;

CONNECTED = 3;

DISCONNECTING = 4;

MongoAdapter = module.exports = function(conf) {
  EventEmitter.call(this);
  this._ver = 0;
  if (conf) this._loadConf(conf);
  this._state = DISCONNECTED;
  this._collections = {};
  this._pending = [];
};

MongoAdapter.prototype = {
  _loadConf: function(conf) {
    var uri, _ref, _ref2, _ref3;
    if (typeof conf === 'string') {
      uri = url.parse(conf);
      this._host = uri.hostname;
      this._port = uri.port || 27017;
      this._database = uri.pathname.replace(/\//g, '');
      return _ref3 = (_ref = (_ref2 = uri.auth) != null ? _ref2.split(':') : void 0) != null ? _ref : [], this._user = _ref3[0], this._pass = _ref3[1], _ref3;
    } else {
      return this._host = conf._host, this._port = conf._port, this._database = conf._database, this._user = conf._user, this._pass = conf._pass, conf;
    }
  },
  connect: function(conf, callback) {
    var _this = this;
    if ('function' === typeof conf) {
      callback = conf;
    } else {
      this._loadConf(conf);
    }
    this._db = new mongo.Db(this._database, new mongo.Server(this._host, this._port));
    this._state = CONNECTING;
    this.emit('connecting');
    return this._db.open(function(err) {
      var open;
      if (err && callback) return callback(err);
      open = function() {
        var todo, _i, _len, _ref;
        _this._state = CONNECTED;
        _this.emit('connected');
        _ref = _this._pending;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          todo = _ref[_i];
          _this[todo[0]].apply(_this, todo[1]);
        }
        return _this._pending = [];
      };
      if (_this._user && _this._pass) {
        return _this._db.authenticate(_this._user, _this._pass, open);
      }
      return open();
    });
  },
  disconnect: function(callback) {
    var _this = this;
    switch (this._state) {
      case DISCONNECTED:
        return callback(null);
      case CONNECTING:
        return this.once('connected', function() {
          return _this.close(callback);
        });
      case CONNECTED:
        this._state = DISCONNECTING;
        this._db.close();
        this._state = DISCONNECTED;
        this.emit('disconnected');
        if (callback) return callback();
        break;
      case DISCONNECTING:
        return this.once('disconnected', function() {
          return callback(null);
        });
    }
  },
  flush: function(callback) {
    if (this._state !== CONNECTED) return this._pending.push(['flush', arguments]);
    return this._db.dropDatabase(function(err, done) {
      if (err) throw err;
      return callback();
    });
  },
  set: function(path, val, ver, callback) {
    var collection, delta, id, _ref;
    if (this._state !== CONNECTED) return this._pending.push(['set', arguments]);
    _ref = path.split('.'), collection = _ref[0], id = _ref[1], path = 3 <= _ref.length ? __slice.call(_ref, 2) : [];
    if (path.length) {
      path = path.join('.');
      delta = {
        ver: ver
      };
      delta[path] = val;
      return this._collection(collection).update({
        _id: id
      }, {
        $set: delta
      }, {
        upsert: true,
        safe: true
      }, callback);
    }
    delta = this._flatten(path.join('.'), val);
    delta.ver = ver;
    return this._collection(collection).update({
      _id: id
    }, {
      $set: delta
    }, {
      upsert: true,
      safe: true
    }, callback);
  },
  del: function(path, ver, callback) {
    var collection, id, unset, _ref;
    if (this._state !== CONNECTED) return this._pending.push(['del', arguments]);
    _ref = path.split('.'), collection = _ref[0], id = _ref[1], path = 3 <= _ref.length ? __slice.call(_ref, 2) : [];
    if (path.length) {
      path = path.join('.');
      unset = {};
      unset[path] = 1;
      return this._collection(collection).update({
        _id: id
      }, {
        $unset: unset
      }, {
        safe: true
      }, callback);
    }
    return this._collection(collection).remove({
      _id: id
    }, callback);
  },
  get: function(path, callback) {
    var collection, fields, id, _ref;
    if (this._state !== CONNECTED) return this._pending.push(['get', arguments]);
    _ref = path.split('.'), collection = _ref[0], id = _ref[1], path = 3 <= _ref.length ? __slice.call(_ref, 2) : [];
    if (path.length) {
      path = path.join('.');
      fields = {
        _id: 0,
        ver: 1
      };
      fields[path] = 1;
      return this._collection(collection).findOne({
        _id: id
      }, {
        fields: fields
      }, function(err, doc) {
        if (err) return callback(err);
        return callback(null, doc && doc[path], doc && doc.ver);
      });
    }
    return this._collection(collection).findOne({
      _id: id
    }, function(err, doc) {
      var ver;
      if (err) return callback(err);
      ver = doc && doc.ver;
      if (doc) delete doc.ver;
      return callback(null, doc, ver);
    });
  },
  _collection: function(name) {
    var _base;
    return (_base = this._collections)[name] || (_base[name] = new Collection(name, this._db));
  },
  _genObjectId: function() {
    return ObjectId.toString(new ObjectId);
  },
  _flatten: function(path, obj, delta) {
    var k, nextPath, v;
    if (delta == null) delta = {};
    if (obj.constructor !== Object) {
      delta[path] = obj;
      return delta;
    }
    for (k in obj) {
      v = obj[k];
      nextPath = path ? "" + path + "." + k : k;
      this._flatten(nextPath, v, delta);
    }
    return delta;
  }
};

MongoAdapter.prototype.__proto__ = EventEmitter.prototype;

MongoCollection = mongo.Collection;

Collection = function(name, db) {
  var self;
  self = this;
  self.name = name;
  self.db = db;
  self._pending = [];
  self._ready = false;
  db.collection(name, function(err, collection) {
    if (err) throw err;
    self._ready = true;
    self.collection = collection;
    return self.onReady();
  });
};

Collection.prototype = {
  onReady: function() {
    var todo, _i, _len, _ref;
    _ref = this._pending;
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      todo = _ref[_i];
      this[todo[0]].apply(this, todo[1]);
    }
    return this._pending = [];
  }
};

_ref = MongoCollection.prototype;
_fn = function(name, fn) {
  return Collection.prototype[name] = function() {
    var args, collection;
    collection = this.collection;
    args = arguments;
    if (this._ready) {
      return process.nextTick(function() {
        return collection[name].apply(collection, args);
      });
    } else {
      return this._pending.push([name, arguments]);
    }
  };
};
for (name in _ref) {
  fn = _ref[name];
  _fn(name, fn);
}
